#include "cb32.h"

GPIO_InitTypeDef GPIO_InitStructure;
USART_InitTypeDef USART_InitStructure;
NVIC_InitTypeDef NVIC_InitStructure;

void NVIC_Configuration(void);

/*******************************************************************************************************
/* SYSTICK_initial	:	SYSTICK_ configuration   					     
/******************************************************************************************************/
void SYSTICK_initial(void) { 	

  /* SysTick end of count event each 1ms with input clock equal to 9MHz (HCLK/8, default) */
  SysTick_SetReload(9000);
  
  /* Configure the SysTick Handler Priority: Preemption priority and subpriority */
  NVIC_SystemHandlerPriorityConfig(SystemHandler_SysTick, 0, 7);  

   /* Enable SysTick interrupt */
  SysTick_ITConfig(ENABLE);

  /* Enable the SysTick Counter */
  SysTick_CounterCmd(SysTick_Counter_Enable);
  
}

void RCC_Configuration(void)
{
  ErrorStatus HSEStartUpStatus;
  /* RCC system reset(for debug purpose) */
  RCC_DeInit();

  /* Enable HSE */
  RCC_HSEConfig(RCC_HSE_ON);

  /* Wait till HSE is ready */
  HSEStartUpStatus = RCC_WaitForHSEStartUp();

  if (HSEStartUpStatus == SUCCESS)
  {
    /* Enable Prefetch Buffer */
    FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable);

    /* Flash 2 wait state */
    FLASH_SetLatency(FLASH_Latency_2);

    /* HCLK = SYSCLK */
    RCC_HCLKConfig(RCC_SYSCLK_Div1);

    /* PCLK2 = HCLK */
    RCC_PCLK2Config(RCC_HCLK_Div1);

    /* PCLK1 = HCLK/4 */
    RCC_PCLK1Config(RCC_HCLK_Div2);

    /* PLLCLK = 8MHz * 9 = 72 MHz */
    RCC_PLLConfig(RCC_PLLSource_HSE_Div1, RCC_PLLMul_9);

    /* Enable PLL */
    RCC_PLLCmd(ENABLE);

    /* Wait till PLL is ready */
    while (RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET)
    {}

    /* Select PLL as system clock source */
    RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);

    /* Wait till PLL is used as system clock source */
    while (RCC_GetSYSCLKSource() != 0x08)
    {}
  }
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM2 | RCC_APB1Periph_TIM3 | RCC_APB1Periph_TIM4 | RCC_APB1Periph_TIM5 |
    RCC_APB1Periph_USART2 | RCC_APB1Periph_USART3 | RCC_APB1Periph_UART4 | RCC_APB1Periph_UART5 | RCC_APB1Periph_SPI2 , ENABLE);

  RCC_APB2PeriphClockCmd(RCC_APB2Periph_ADC1 | RCC_APB2Periph_GPIOA | RCC_APB2Periph_GPIOB| RCC_APB2Periph_GPIOC| 
    RCC_APB2Periph_GPIOD| RCC_APB2Periph_GPIOE| RCC_APB2Periph_TIM1 | RCC_APB2Periph_TIM8 | 
    RCC_APB2Periph_AFIO | RCC_APB2Periph_USART1 |  
    RCC_APB2Periph_GPIOF| RCC_APB2Periph_GPIOG, ENABLE);
}

/*******************************************************************************
* Function Name  : NVIC_Configuration
*******************************************************************************/
void NVIC_Configuration(void)
{
#ifdef  VECT_TAB_RAM
  /* Set the Vector Table base location at 0x20000000 */
  NVIC_SetVectorTable(NVIC_VectTab_RAM, 0x0);
#else  /* VECT_TAB_FLASH  */
  /* Set the Vector Table base location at 0x08000000 */
  NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x0);
#endif 

  /* Enable the TIM5 gloabal Interrupt */
  NVIC_InitStructure.NVIC_IRQChannel = TIM5_IRQChannel;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 5;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);

  /* Enable the TIM1 update Interrupt */
  NVIC_InitStructure.NVIC_IRQChannel = TIM1_UP_IRQChannel;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 5;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);
  
  /* Enable the USART2 Interrupt */
  NVIC_InitStructure.NVIC_IRQChannel = USART2_IRQChannel;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 4;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);
  
  /* Enable the USART3 Interrupt */
  NVIC_InitStructure.NVIC_IRQChannel = USART3_IRQChannel;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 3;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);

  /* Enable the UART4 Interrupt */
  NVIC_InitStructure.NVIC_IRQChannel = UART4_IRQChannel;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 2;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);

  /* Enable the UART5 Interrupt */
  NVIC_InitStructure.NVIC_IRQChannel = UART5_IRQChannel;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 1;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);

}

int __io_putchar(int ch)
{
  // Write a character to the USART 
  USART_SendData(USART1, (u8) ch);

  // Loop until the end of transmission 
  while(USART_GetFlagStatus(USART1, USART_FLAG_TXE) == RESET);

  return ch;
}

/*******************************************************************************
*  Main
*******************************************************************************/

// simple MODBUS  MODBUS ٽ κи ߷ MCU    ֵ  ̴.

// simple MODBUS  ʿ ũ  . ݵ static  Ѵ. 
// unsiged 8Ʈ 迭 Ʈ  ǰ
// unsigned 16Ʈ 迭   ȴ.

// HMI   ʹ ̰ ǹǷ, ٸ ƾ   ؼ α׷ ۼؾ Ѵ.
// ݴ, HMI ϴ ͵ ̰ ϸ ȴ. HMI ش  ϰ ȴ.

// HMI 00001 ̶ Ǿ , Ʈ  0 Ѵ.  00001 MDcoil[0] 0Ʈ̴. 
// 00002 MDcoil[0] 1Ʈ ̴. ̷ 00008 MDcoil[1] 0 Ʈ ȴ.

// HMI 40001 ̶ Ǿ  念 0 Ѵ.  40001 MDregister[0] ̴. 
// տ 4 <念> ϸ ڿ 4ڸ  ̴ּ.

//  迭 ũ ʿ信  ̰ų ϼ ִ. 

static u8 MDcoil[100];
static u16 MDregister[100];

//
//  巹  Ʈ ¸ о Լ 
// a = coil(9); // ̷ ϸ 00009 Ʈ 巹ġ ãƼ ش Ʈ о´.

u8 MDbitDecode[8] = {1,2,4,8,16,32,64,128};

u8 coil(u16 MDBSadr)
{
    MDBSadr--;
    if (MDcoil[MDBSadr>>3] & MDbitDecode[MDBSadr & 7]) return 1;
    return 0;
}

//
//  巹 Ʈ ¸ 
// coilSet(9,0); // 00009巹 0 .
// coilSet(9,1); // 00009巹 1 . 
//

void coilSet(u16 MDBSadr, u8 MDBSdata)
{
    u16 arrayindex;
    MDBSadr--;
    arrayindex = MDBSadr>>3; // 迭 ġ 
    if (MDBSdata==0) {
        MDcoil[arrayindex] &= ~MDbitDecode[MDBSadr & 7];
        }
    else {
        MDcoil[arrayindex] |= MDbitDecode[MDBSadr & 7];
        }
}

//
//  巹 () о´.
//  巹 ܼ 40001  迭 ġ ȴ. 
//

u16 getReg(u16 MDBSadr)
{
    return MDregister[MDBSadr - 40001];
}

//
//  巹 () ִ´.
//  巹 ܼ 40001  迭 ġ ȴ. 
//


void setReg(u16 MDBSadr, u16 MDBSdata)
{
    MDregister[MDBSadr - 40001] = MDBSdata;
}

// simple MODBUS ó  2 ͷƮ Ѵ.
// ϳ UART ͷƮε, UARTͰ ö ߻Ѵ.  ŵǸ ۿ ϰ,  ͸ ؼ
// simple MODBUS  ϼθ üũѴ. ۿ ִ   CRC ġ ϸ, __modbusFoundFrame  1  ش.

// Ǵٸ ϳ systick Ÿ̸ ͷƮε,  α׷ 1mS ѹ ߻ϵ سҴ.
// ̰ __modbusFoundFrame üũؼ 1 Ǿ , ۿ ִ ͸ мؼ óϰ Ѵ.

// Ʒ main  simple MODBUS   UARTƮ ϰ, startSimpleModbus ȣؼ simple MODBUS ׶忡
// ư  ֵϸ ָ ȴ. ׷ ռ  ͷƮ systickͷƮ ˾Ƽ simple MODBUS  óش.

//  ũ 迭 (MDcoil, MDregister) ؼ HMI ͸ ְ޴  ڵ ָ ȴ. 

int main(void)
{
    /* System Clocks Configuration */
    RCC_Configuration();
    //PLL_initial();
    /* NVIC Configuration */
    NVIC_Configuration();
    SYSTICK_initial();

    openCom(0,115200, C8N1);  // ä0 115200, 8Ʈ noneиƼ, 1Ʈ : MCU UART2 Ǿ ִ.
    startSimpleModbus(0,1,MDregister, MDcoil); // simple MODBUS   : ä0, ̺巹1, 迭, Ʈ迭

    // User code area (ʱȭ)

    u16 basetime=0;
    for(;;) {
    
        // User code area (κ)
    
    
        // 200mS ѹ  
        basetime++;
        if (basetime > 100) {
            //MDregister[0]++;  // HMI 40001 о ǥ
            setReg(40001,getReg(40001)+1); // 40001 1  
            MDcoil[0]++;
           basetime=0;
            }
    
        // 00009 1̸ status led  on
        if (coil(9) == 0) statusLed(0);
            else statusLed(1);
 
        // 00012 1̸ status led  on
        if (coil(12) == 0) clcdPower(0);
            else clcdPower(1);
 
        // 00011 1̸ 40002 1
        if (coil(11) == 0) MDregister[1]++;

        // keyinput
        portMode(81,__IN);
        if (portIn(81) != 0) coilSet(10,0); // bit 1 (巹 00010) off
            else coilSet(10,1); // bit 1 on
            
        delay(1);
        }
      
}

/*

 α׷  ϱ ϵ  Լ ϰ  

---openCom
void  openCom (u8 comCh,  u32 comBaud,  u8 comMode) 

  - comCh : äιȣ
  - comBaud :  Ʈ
  - comMode :   

ش ä  Ʈ    OPEN  մϴ.
comBaud   Ʈ  ݴϴ.  300, ִ 115200  , ƹ̳ ϴ Ʈ ָ˴ϴ. 
Ϲ  Ǵ Ʈ  ϴ.
1200, 2400,  4800,  9600,  14400,  19200,  28800,  38400,  57600,  76800,  115200

comMode  8Ʈ noneиƼ, 1 Ʈ C8N1   ־ մϴ. ϳ ϼ. 

C7E1 : 7Ʈ, EVENиƼ, 1 STOPƮ
C7O1 : 7Ʈ, ODDиƼ, 1 STOPƮ
C8N1 : 8Ʈ, NOиƼ, 1 STOPƮ
C8E1 : 8Ʈ, EVENиƼ, 1 STOPƮ
C8O1 : 8Ʈ, ODDиƼ, 1 STOPƮ

C7E2 : 7Ʈ, EVENиƼ, 2 STOPƮ
C7O2 : 7Ʈ, ODDиƼ, 2 STOPƮ
C8N2 : 8Ʈ, NOиƼ, 2 STOPƮ
C8E2 : 8Ʈ, EVENиƼ, 2 STOPƮ
C8O2 : 8Ʈ, ODDиƼ, 2 STOPƮ

OpenCom 255Ʈ Ź۰ ҴǸ, ش۴ Ŭ˴ϴ.
Ź ũ⸦ ڰ   ϴ.

(뿹)
openCom(0, 115200, com8N1);


---startSimpleModbus
void startSimpleModbus(u8 comCh, u8 modSlaveadr, u16 * modbusBufferRegister, u8 * modbusBufferCoil)

  - comCh :  ä (0Ǵ 1 Ǵ 2 밡)
  - modSlaveadr :  ̺ 巹
  - modbusBufferRegister :   ۹
  - modbusBufferCoil :   ۹ 

Simple MODBUS  ˸ ԼԴϴ. openComԼ ٷ Ʒ ۼ ݴϴ.

(뿹)
  u8 MDcoil[10];
  u16 MDregister[10];
  openCom(2,38400, com8N1);
  startSimpleModbus(2,1,MDregister, MDcoil);


---comPut
void comPut (u8  comCh,  u8 comChar) 

  - comCh : äιȣ
  - comChar :  ۽ĳ

ѹƮ ۽ϴ Լ Դϴ. comCh ä ȣ, comChar  1Ʈ  (ASCIIڵ Ǵ ) ݴϴ. 

(뿹)
openCom(1, 115200, com8N1);
while (1) {
	delay(500);
	comPut(1,0x41);
	comPut(1,'B');
	}	

---comGet
short  comGet (u8 comCh) 

  - comCh : äιȣ

Źۿ ĳ ѹƮ о Լ Դϴ. comCh ä ȣ ݴϴ.
Ʈκ ŵ Ʈ ο ִ 255Ʈ Ź۷ ڵ˴ϴ.  comGetԼ   ŵ Ʈ оɴϴ.

(뿹)
openCom(0, 115200, com8N1);
while (1) {
	while (!comLen(0));  // Ŵ
	chari = comGet(0);   // ѹƮ 
	comPut(0,chari);     // ״ ۽
	}	

---comLen
u16  comLen (u8  comCh) 

  - comCh : äιȣ

Źۿ ׿ִ ͼ ȯִ ԼԴϴ. ƹ ͵ ŵǾ  ʴٸ 0 ȯմϴ.

(뿹)
openCom(0, 115200, com8N1);
while (1) {
	while (!comLen(0));  // Ŵ
	chari = comGet(0);   // ѹƮ 
	comPut(0,chari);     // ״ ۽
	}	

---comFlush
u16  comFlush (u8  comCh) 

comCh : äιȣ

Ź۸  ûִ ԼԴϴ. 

(뿹)
comFlush(0); //0 ä Ź۸  Ŭ



*/