#include "cb32.h"
GPIO_InitTypeDef GPIO_InitStructure;

/*
    PORTA : 0x00 to 0x0f
    PORTB : 0x10 to 0x1f
    PORTC : 0x20 to 0x2f
    PORTD : 0x30 to 0x3f
    PORTE : 0x40 to 0x4f
    PORTF : 0x50 to 0x5f
    PORTG : 0x60 to 0x6f
*/

const u16 __ncdtb[] = {1,2,4,8,0x10,0x20,0x40,0x80,0x100,0x200,0x400,0x800,0x1000,0x2000,0x4000,0x8000};

const u8 __iomaptb[] = {
  0x6e,0x6f,0x6c,0x6d,0x6a,0x6b,0x37,0x69, 0xff,0xff,
  0x50,0x51,0x52,0x53,0x54,0x55,0x56,0x57, 0xff,0xff,
  0x35,0x36,0x33,0x34,0x30,0x31,0x04,0x05, 0xff,0xff,
  0x58,0x59,0x60,0x61,0x47,0x48,0x49,0x4a, 0xff,0xff,
  0x08,0x20,0x67,0x68,0x3b,0x3c,0x3d,0x3e, 0xff,0xff,
  0x4b,0x4c,0x4d,0x4e,0x4f,0x38,0x39,0x3a, 0xff,0xff,

  0x5a,0x5b,0x5c,0x5d,0x5e,0x5f,0x46,0x2e,0x2d,0x2f,
  0x3f,0x31,0x32,0x33,0x18,0x19,0x62,0x63,0x40,0x41,
  0x42,0x43,0x44,0x45,0x64,0x65,0x66,0x1c,0x1d,0x1e,
  0x1f
  };

void portMode(u16 portnumber, u8 outputmode)
{
    u16 bitpos;
    portnumber = __iomaptb[portnumber];
    bitpos = portnumber & 0xf;
    GPIO_InitStructure.GPIO_Pin = __ncdtb[bitpos];
    switch (outputmode) {
    case __OUT:
        GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
        break;
    case __OUTOD:
        GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_OD;
        break;
    case __IN:
        GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
        break;
    case __INPD:
        GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPD;
        break;
    case __INPU:
        GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;
        break;
    }
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    
    switch (portnumber >> 4) {
    case 0: //PORTA
        GPIO_Init(GPIOA, &GPIO_InitStructure);
        break;
    case 1: //PORTB
        GPIO_Init(GPIOB, &GPIO_InitStructure);
        break;
    case 2: //PORTC
        GPIO_Init(GPIOC, &GPIO_InitStructure);
        break;
    case 3: //PORTD
        GPIO_Init(GPIOD, &GPIO_InitStructure);
        break;
    case 4: //PORTE
        GPIO_Init(GPIOE, &GPIO_InitStructure);
        break;
    case 5: //PORTF
        GPIO_Init(GPIOF, &GPIO_InitStructure);
        break;
    case 6: //PORTG
        GPIO_Init(GPIOG, &GPIO_InitStructure);
        break;
    }
}

void portLow(u16 portnumber)
{
    u16 bitpos;
    portnumber = __iomaptb[portnumber];
    bitpos = portnumber & 0xf;
    bitpos = __ncdtb[bitpos];
    switch (portnumber >> 4) {
    case 0: //PORTA
        GPIO_ResetBits(GPIOA, bitpos);
        break;
    case 1: //PORTB
        GPIO_ResetBits(GPIOB, bitpos);
        break;
    case 2: //PORTC
        GPIO_ResetBits(GPIOC, bitpos);
        break;
    case 3: //PORTD
        GPIO_ResetBits(GPIOD, bitpos);
        break;
    case 4: //PORTE
        GPIO_ResetBits(GPIOE, bitpos);
        break;
    case 5: //PORTF
        GPIO_ResetBits(GPIOF, bitpos);
        break;
    case 6: //PORTG
        GPIO_ResetBits(GPIOG, bitpos);
        break;
    }
}    

void portHigh(u16 portnumber)
{
    u16 bitpos;
    portnumber = __iomaptb[portnumber];
    bitpos = portnumber & 0xf;
    bitpos = __ncdtb[bitpos];
    switch (portnumber >> 4) {
    case 0: //PORTA
        GPIO_SetBits(GPIOA, bitpos);
        break;
    case 1: //PORTB
        GPIO_SetBits(GPIOB, bitpos);
        break;
    case 2: //PORTC
        GPIO_SetBits(GPIOC, bitpos);
        break;
    case 3: //PORTD
        GPIO_SetBits(GPIOD, bitpos);
        break;
    case 4: //PORTE
        GPIO_SetBits(GPIOE, bitpos);
        break;
    case 5: //PORTF
        GPIO_SetBits(GPIOF, bitpos);
        break;
    case 6: //PORTG
        GPIO_SetBits(GPIOG, bitpos);
        break;
    }
}    

void portOut(u16 portnumber, u8 value)
{
    u16 bitpos;
    portnumber = __iomaptb[portnumber];
    bitpos = portnumber & 0xf;
    bitpos = __ncdtb[bitpos];
    switch (portnumber >> 4) {
    case 0: //PORTA
        GPIO_WriteBit(GPIOA, bitpos, value);
        break;
    case 1: //PORTB
        GPIO_WriteBit(GPIOB, bitpos, value);
        break;
    case 2: //PORTC
        GPIO_WriteBit(GPIOC, bitpos, value);
        break;
    case 3: //PORTD
        GPIO_WriteBit(GPIOD, bitpos, value);
        break;
    case 4: //PORTE
        GPIO_WriteBit(GPIOE, bitpos, value);
        break;
    case 5: //PORTF
        GPIO_WriteBit(GPIOF, bitpos, value);
        break;
    case 6: //PORTG
        GPIO_WriteBit(GPIOG, bitpos, value);
        break;
    }
}    

void portReverse(u16 portnumber)
{
    u16 bitpos;
    portnumber = __iomaptb[portnumber];
    bitpos = portnumber & 0xf;
    bitpos = __ncdtb[bitpos];
    switch (portnumber >> 4) {
    case 0: //PORTA
        GPIO_WriteBit(GPIOA, bitpos, (BitAction)(1 - GPIO_ReadOutputDataBit(GPIOA, bitpos)));
        break;
    case 1: //PORTB
        GPIO_WriteBit(GPIOB, bitpos, (BitAction)(1 - GPIO_ReadOutputDataBit(GPIOB, bitpos)));
        break;
    case 2: //PORTC
        GPIO_WriteBit(GPIOC, bitpos, (BitAction)(1 - GPIO_ReadOutputDataBit(GPIOC, bitpos)));
        break;
    case 3: //PORTD
        GPIO_WriteBit(GPIOD, bitpos, (BitAction)(1 - GPIO_ReadOutputDataBit(GPIOD, bitpos)));
        break;
    case 4: //PORTE
        GPIO_WriteBit(GPIOE, bitpos, (BitAction)(1 - GPIO_ReadOutputDataBit(GPIOE, bitpos)));
        break;
    case 5: //PORTF
        GPIO_WriteBit(GPIOF, bitpos, (BitAction)(1 - GPIO_ReadOutputDataBit(GPIOF, bitpos)));
        break;
    case 6: //PORTG
        GPIO_WriteBit(GPIOG, bitpos, (BitAction)(1 - GPIO_ReadOutputDataBit(GPIOG, bitpos)));
        break;
    }
}    

u8 portIn(u16 portnumber)
{
    u16 bitpos;
    portnumber = __iomaptb[portnumber];
    bitpos = portnumber & 0xf;
    bitpos = __ncdtb[bitpos];
    switch (portnumber >> 4) {
    case 0: //PORTA
        return GPIO_ReadInputDataBit(GPIOA, bitpos);
    case 1: //PORTB
        return GPIO_ReadInputDataBit(GPIOB, bitpos);
    case 2: //PORTC
        return GPIO_ReadInputDataBit(GPIOC, bitpos);
    case 3: //PORTD
        return GPIO_ReadInputDataBit(GPIOD, bitpos);
    case 4: //PORTE
        return GPIO_ReadInputDataBit(GPIOE, bitpos);
    case 5: //PORTF
        return GPIO_ReadInputDataBit(GPIOF, bitpos);
    case 6: //PORTG
        return GPIO_ReadInputDataBit(GPIOG, bitpos);
    }
}    

u8 portOutStat(u16 portnumber)
{
    u16 bitpos;
    portnumber = __iomaptb[portnumber];
    bitpos = portnumber & 0xf;
    bitpos = __ncdtb[bitpos];
    switch (portnumber >> 4) {
    case 0: //PORTA
        return GPIO_ReadOutputDataBit(GPIOA, bitpos);
    case 1: //PORTB
        return GPIO_ReadOutputDataBit(GPIOB, bitpos);
    case 2: //PORTC
        return GPIO_ReadOutputDataBit(GPIOC, bitpos);
    case 3: //PORTD
        return GPIO_ReadOutputDataBit(GPIOD, bitpos);
    case 4: //PORTE
        return GPIO_ReadOutputDataBit(GPIOE, bitpos);
    case 5: //PORTF
        return GPIO_ReadOutputDataBit(GPIOF, bitpos);
    case 6: //PORTG
        return GPIO_ReadOutputDataBit(GPIOG, bitpos);
    }
}    

void statusLed(u8 onoff)
{
  portMode(86,__OUT);
  portOut(86,onoff);
}

void comPower(u8 onoff)
{
  portMode(84,__OUT);
  if (onoff) 
    portLow(84);
  else
    portHigh(84);
}

void clcdPower(u8 onoff)
{
  portMode(85,__OUT);
  if (onoff) 
    portLow(85);
  else
    portHigh(85);
}

// delay functions

void delaymicro(vu32 nCountus)
{
    for (;nCountus > 0;nCountus--);
}

void delay(vu32 nCount)
{
    for (;nCount > 0;nCount--) {
        delaymicro(0x1598);
        }
}

void delayUs(vu32 nCount)
{
    for (;nCount > 0;nCount--) {
        delaymicro(2);
        }
}

